/*
 * ##################################################
 * #                                                #
 * # rkTicTacToe                                    #
 * # The classic cult game                          #
 * #                                                #
 * # Copyright (C) by René Knipschild               #
 * #                                                #
 * #                                email@rkcsd.com #
 * #                         www.reneknipschild.net #
 * #                                                #
 * ##################################################
 *
 * File: Main.java - core
 * Version: 1.0.0
 * Last modified: 2011/04/21 10:18 CEST
 * Author: rkCSD (go.reneknipschild.net/com)
 *
 * ===Notes==========================================
 * There are currently no notes.
 * ==================================================
 */

package com.rkcsd.apps.desktoptools.tictactoe.core;

import java.awt.Font;
import java.util.Enumeration;

import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;
import javax.swing.plaf.FontUIResource;

import com.rkcsd.apps.desktoptools.tictactoe.gui.UserInterface;

public class Main {
	public static void main(String[] args) {
		printInitMessage();
		if (args.length > 0 && args[0].equals("--nosyslaf")) {
			init(false);
		} else {
			init(true);
		}
		new UserInterface();
	}
	
	public static void printInitMessage() {
		System.out.println("rkTicTacToe (Version 1.1)");
		System.out.println();
		System.out.println("Copyright © 2009 René Knipschild <email@rkcsd.com>");
		System.out.println("Visit my website: http://www.reneknipschild.net/");
		System.out.println();
		System.out.println("This program is free software; you can redistribute it and/or modify");
		System.out.println("it under the terms of the GNU General Public License as published by");
		System.out.println("the Free Software Foundation; either version 2 of the License, or");
		System.out.println("(at your option) any later version.");
		System.out.println();
		System.out.println("This program is distributed in the hope that it will be useful,");
		System.out.println("but WITHOUT ANY WARRANTY; without even the implied warranty of");
		System.out.println("MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the");
		System.out.println("GNU General Public License for more details.");
		System.out.println();
		System.out.println("You should have received a copy of the GNU General Public License");
		System.out.println("along with this program; if not, write to the Free Software");
		System.out.println("Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,");
		System.out.println("MA 02110-1301, USA.");
	}
	
	public static void init(boolean useSystemLAF) {
		int requiredJavaVersion = Integer.parseInt("1.6".substring(2, 3));
		int currentJavaVersion = Integer.parseInt(System.getProperty("java.runtime.version").substring(2, 3));
		if (currentJavaVersion < requiredJavaVersion) {
			System.err.println("Fatal error:\n\trkTicTacToe requires Java Runtime Version 1.6 or later.\n\tYou run Java Runtime Version " + System.getProperty("java.runtime.version").substring(0, 3) + ".\n\tYou need to upgrade if you want to run rkTicTacToe.\n\tFor support send your question to email@rkcsd.com or call +49 5631 9189488.");
			System.exit(1);
		}
		JFrame.setDefaultLookAndFeelDecorated(true);
		JDialog.setDefaultLookAndFeelDecorated(true);
		if (useSystemLAF) {
			try {
				UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
			} catch (UnsupportedLookAndFeelException exception) {
				exception.printStackTrace();
			} catch (ClassNotFoundException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (InstantiationException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (IllegalAccessException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		UIManager.put("AuditoryCues.playList", UIManager.get("AuditoryCues.allAuditoryCues"));
		for (Enumeration<?> e = UIManager.getDefaults().keys(); e.hasMoreElements();) {
			Object key = e.nextElement();
			Object value = UIManager.get(key);
			if (value instanceof Font) {
				Font font = (Font) value;
				if (!key.toString().equals("InternalFrame.titleFont")) {
					UIManager.put(key, new FontUIResource(font.getName(), Font.PLAIN, 11));
				} else {
					UIManager.put(key, new FontUIResource(font.getName(), Font.BOLD, 12));
				}
			}
		}
	}
}