/*
 * ##################################################
 * #                                                #
 * # rkTicTacToe                                    #
 * # The classic cult game                          #
 * #                                                #
 * # Copyright (C) by René Knipschild               #
 * #                                                #
 * #                                email@rkcsd.com #
 * #                         www.reneknipschild.net #
 * #                                                #
 * ##################################################
 *
 * File: UserInterface.java - gui
 * Version: 1.0.0
 * Last modified: 2011/04/21 10:48 CEST
 * Author: rkCSD (go.reneknipschild.net/com)
 *
 * ===Notes==========================================
 * There are currently no notes.
 * ==================================================
 */

package com.rkcsd.apps.desktoptools.tictactoe.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridLayout;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.JTextField;
import javax.swing.SpinnerNumberModel;
import javax.swing.SwingUtilities;
import javax.swing.WindowConstants;

import com.rkcsd.apps.desktoptools.tictactoe.tools.Browser;
import com.rkcsd.apps.desktoptools.tictactoe.tools.GameFieldValidator;

public class UserInterface {
	private JFrame window;

	private JPanel frame;

	private int fields;

	private String[] signs;

	private JButton[][] field;

	private int sign;

	private JPanel options;

	private JLabel fieldsLabel;

	private JSpinner fieldsInput;

	private JLabel signsLabel;

	private JTextField signsInput;

	private JButton confirmButton;

	private JPanel game;

	private JLabel copyright;

	private Color defaultColor = new JButton().getForeground();

	private Color grayedColor = Color.GRAY;

	public UserInterface() {
		this.window = new JFrame("rkTicTacToe");
		this.window.setIconImage(Toolkit.getDefaultToolkit().getImage(ClassLoader.getSystemResource("com/rkcsd/apps/desktoptools/tictactoe/resources/rkIcon.png")));
		this.window.setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
		this.frame = new JPanel(new BorderLayout());
		this.frame.setBorder(BorderFactory.createEmptyBorder(4, 4, 4, 4));

		this.options = new JPanel();
		ComfortGridBagLayout layout = new ComfortGridBagLayout();
		this.game = new JPanel();
		this.fieldsLabel = new JLabel("Fields:");
		this.fieldsInput = new JSpinner(new SpinnerNumberModel(3, 1, 25, 1));
		this.signsLabel = new JLabel("Signs:");
		this.signsInput = new JTextField("X;O");
		this.confirmButton = new JButton("Start");
		this.copyright = new JLabel("<html><p>Copyright © 2009 René Knipschild (<a href=\"http://www.reneknipschild.net/\">www.reneknipschild.net</a>)</p></html>)");

		this.options.setLayout(layout);
		this.game.setBorder(BorderFactory.createEmptyBorder(4, 4, 4, 4));
		this.game.setPreferredSize(new Dimension(400, 300));
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				UserInterface.this.confirmButton.requestFocusInWindow();
			}
		});
		this.copyright.setBorder(BorderFactory.createEmptyBorder(0, 4, 4, 4));

		layout.createGridBagConstraints(this.fieldsLabel, 0, 0, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, 0, 0, 0, 0);
		layout.createGridBagConstraints(this.fieldsInput, 1, 0, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, 1, 0, 0, 0);
		layout.createGridBagConstraints(this.signsLabel, 2, 0, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, 0, 0, 0, 0);
		layout.createGridBagConstraints(this.signsInput, 3, 0, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, 10, 0, 0, 0);
		layout.createGridBagConstraints(this.confirmButton, 4, 0, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, 0, 0, 25, 0);

		this.confirmButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				UserInterface.this.start(Integer.parseInt(UserInterface.this.fieldsInput.getValue().toString()), UserInterface.this.signsInput.getText().split(";"));
			}
		});

		this.copyright.addMouseListener(new MouseListener() {
			public void mouseClicked(MouseEvent e) {
				Browser.open("http://www.reneknipschild.net/");
			}

			public void mousePressed(MouseEvent e) {

			}

			public void mouseReleased(MouseEvent e) {

			}

			public void mouseEntered(MouseEvent e) {
				UserInterface.this.copyright.setCursor(new Cursor(Cursor.HAND_CURSOR));
			}

			public void mouseExited(MouseEvent e) {
				UserInterface.this.copyright.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
			}
		});

		this.options.add(this.fieldsLabel);
		this.options.add(this.fieldsInput);
		this.options.add(this.signsLabel);
		this.options.add(this.signsInput);
		this.options.add(this.confirmButton);

		this.frame.add(this.options, BorderLayout.NORTH);
		this.frame.add(this.game, BorderLayout.CENTER);
		this.frame.add(this.copyright, BorderLayout.SOUTH);

		this.window.add(this.frame, BorderLayout.CENTER);
		this.window.pack();
		this.window.setResizable(true);
		this.window.setLocationRelativeTo(null);
		this.window.setVisible(true);
	}

	private void start(int fields, String[] signs) {
		this.sign = 0;
		this.fields = fields;
		this.signs = signs;
		this.game.removeAll();
		this.field = new JButton[this.fields][this.fields];
		this.game.setLayout(new GridLayout(this.fields, this.fields, 8, 8));
		for (int i = 0; i < this.fields; i++) {
			for (int j = 0; j < this.fields; j++) {
				this.field[i][j] = new JButton();
				this.field[i][j].addMouseListener(new MouseListener() {
					public void mouseClicked(MouseEvent event) {

					}

					public void mouseEntered(MouseEvent event) {
						UserInterface.this.hover(event, true);
					}

					public void mouseExited(MouseEvent event) {
						UserInterface.this.hover(event, false);
					}

					public void mousePressed(MouseEvent event) {

					}

					public void mouseReleased(MouseEvent event) {

					}
				});
				this.field[i][j].addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent event) {
						UserInterface.this.click(event);
					}
				});
				this.field[i][j].setForeground(this.defaultColor);
				this.field[i][j].setFont(this.field[i][j].getFont().deriveFont(Font.BOLD, 24f));
				this.game.add(this.field[i][j]);
			}
		}
		this.game.updateUI();
	}

	private void hover(MouseEvent event, boolean hover) {
		int x = this.getField(event.getSource(), 0);
		int y = this.getField(event.getSource(), 1);
		if (hover) {
			if (this.field[x][y].getText().equals("") && this.field[x][y].getForeground().equals(this.defaultColor)) {
				this.field[x][y].setForeground(this.grayedColor);
				this.field[x][y].setText(this.signs[this.sign]);
			}
		} else {
			if (this.field[x][y].getText().equals(this.signs[this.sign]) && this.field[x][y].getForeground().equals(this.grayedColor)) {
				this.field[x][y].setForeground(this.defaultColor);
				this.field[x][y].setText("");
			}
		}
	}

	private void click(ActionEvent event) {
		int x = this.getField(event.getSource(), 0);
		int y = this.getField(event.getSource(), 1);
		if (this.field[x][y].getForeground().equals(this.grayedColor)) {
			this.field[x][y].setText(this.signs[this.sign]);
			this.field[x][y].setForeground(this.defaultColor);
			this.changeSign();
			String[][] plainField = new String[field.length][field[0].length];
			for (int i = 0; i < field.length; i++) {
				for (int j = 0; j < field[0].length; j++) {
					plainField[i][j] = field[i][j].getText();
				}
			}
			String winner = GameFieldValidator.check(signs, plainField, fields);
			if (winner.equals(" ")) {
				JOptionPane.showMessageDialog(this.window, "No winner.", "Information", JOptionPane.INFORMATION_MESSAGE);
				this.reset();
			} else if (!winner.equals("")) {
				JOptionPane.showMessageDialog(this.window, "Player " + winner + " is winner.", "Information", JOptionPane.INFORMATION_MESSAGE);
				this.reset();
			}
		} else {
			JOptionPane.showMessageDialog(this.window, "This field is already in use.", "Error", JOptionPane.ERROR_MESSAGE);
		}
	}

	private int getField(Object source, int mode) {
		int x = 0;
		int y = 0;
		for (int i = 0; i < this.fields; i++) {
			for (int j = 0; j < this.fields; j++) {
				if (source.equals(this.field[i][j])) {
					x = i;
					y = j;
				}
			}
		}
		if (mode == 0) {
			return x;
		} else if (mode == 1) {
			return y;
		}
		return 0;
	}

	private void changeSign() {
		if (this.sign < this.signs.length - 1) {
			this.sign++;
		} else {
			this.sign = 0;
		}
	}

	private void reset() {
		this.sign = 0;
		for (int i = 0; i < this.fields; i++) {
			for (int j = 0; j < this.fields; j++) {
				this.field[i][j].setText("");
				this.field[i][j].setForeground(this.defaultColor);
			}
		}
	}
}